@@ -20,6 +20,7 @@ from .accountscout import AccountScout as AccountScout
 from .accountscout import ScoutReport as ScoutReport
 from .addressableaccount import AddressableAccount as AddressableAccount
 from .arguments import parse_args as parse_args
+from .arguments import setup_logging as setup_logging
 from .balancesheet import BalanceSheet as BalanceSheet
 from .cache import Cache as Cache
 from .cache import MarketCache as MarketCache
@@ -27,6 +28,7 @@ from .cache import PerpMarketCache as PerpMarketCache
 from .cache import PriceCache as PriceCache
 from .cache import RootBankCache as RootBankCache
 from .client import BetterClient as BetterClient
+from .client import ClusterUrlData as ClusterUrlData
 from .client import BlockhashNotFoundException as BlockhashNotFoundException
 from .client import ClientException as ClientException
 from .client import CompoundException as CompoundException
