@@ -100,7 +100,8 @@ class Account(AddressableAccount):
                  in_margin_basket: typing.Sequence[bool],
                  slot_indices: typing.Sequence[bool],
                  base_slots: typing.Sequence[AccountSlot],
-                 msrm_amount: Decimal, being_liquidated: bool, is_bankrupt: bool) -> None:
+                 msrm_amount: Decimal, being_liquidated: bool, is_bankrupt: bool,
+                 advanced_orders: PublicKey, not_upgradable: bool, delegate: PublicKey) -> None:
         super().__init__(account_info)
         self.version: Version = version
 
@@ -116,6 +117,9 @@ class Account(AddressableAccount):
         self.msrm_amount: Decimal = msrm_amount
         self.being_liquidated: bool = being_liquidated
         self.is_bankrupt: bool = is_bankrupt
+        self.advanced_orders: PublicKey = advanced_orders
+        self.not_upgradable: bool = not_upgradable
+        self.delegate: PublicKey = delegate
 
     @property
     def shared_quote_token(self) -> Token:
@@ -263,8 +267,13 @@ class Account(AddressableAccount):
         msrm_amount: Decimal = layout.msrm_amount
         being_liquidated: bool = bool(layout.being_liquidated)
         is_bankrupt: bool = bool(layout.is_bankrupt)
+        advanced_orders: PublicKey = layout.advanced_orders
+        not_upgradable: bool = bool(layout.not_upgradable)
+        delegate: PublicKey = layout.delegate
 
-        return Account(account_info, version, meta_data, group.name, group.address, owner, info, quote, in_margin_basket, active_in_basket, slots, msrm_amount, being_liquidated, is_bankrupt)
+        return Account(account_info, version, meta_data, group.name, group.address, owner, info, quote,
+                       in_margin_basket, active_in_basket, slots, msrm_amount, being_liquidated, is_bankrupt,
+                       advanced_orders, not_upgradable, delegate)
 
     @staticmethod
     def parse(account_info: AccountInfo, group: Group, cache: Cache) -> "Account":
@@ -335,6 +344,34 @@ class Account(AddressableAccount):
             accounts += [account]
         return accounts
 
+    @staticmethod
+    def load_all_for_delegate(context: Context, delegate: PublicKey, group: Group) -> typing.Sequence["Account"]:
+        # mango_group is just after the METADATA, which is the first entry.
+        group_offset = layouts.METADATA.sizeof()
+        # delegate is a PublicKey which is 32 bytes that ends 5 bytes before the end of the layout
+        delegate_offset = layouts.MANGO_ACCOUNT.sizeof() - 37
+        filters = [
+            MemcmpOpts(
+                offset=group_offset,
+                bytes=encode_key(group.address)
+            ),
+            MemcmpOpts(
+                offset=delegate_offset,
+                bytes=encode_key(delegate)
+            )
+        ]
+
+        results = context.client.get_program_accounts(
+            context.mango_program_address, memcmp_opts=filters, data_size=layouts.MANGO_ACCOUNT.sizeof())
+        cache: Cache = group.fetch_cache(context)
+        accounts: typing.List[Account] = []
+        for account_data in results:
+            address = PublicKey(account_data["pubkey"])
+            account_info = AccountInfo._from_response_values(account_data["account"], address)
+            account = Account.parse(account_info, group, cache)
+            accounts += [account]
+        return accounts
+
     @staticmethod
     def load_for_owner_by_address(context: Context, owner: PublicKey, group: Group, account_address: typing.Optional[PublicKey]) -> "Account":
         if account_address is not None:
@@ -715,9 +752,12 @@ class Account(AddressableAccount):
         return f"""« Account {info}, {self.version} [{self.address}]
     {self.meta_data}
     Owner: {self.owner}
+    Delegated To: {self.delegate}
     Group: « Group '{self.group_name}' [{self.group_address}] »
+    Advanced Orders Account: {self.advanced_orders}
     MSRM: {self.msrm_amount}
     Bankrupt? {self.is_bankrupt}
+    Upgradable? {not self.not_upgradable}
     Being Liquidated? {self.being_liquidated}
     Shared Quote Token:
         {shared_quote}
