@@ -160,6 +160,22 @@ class SerumEventQueue(AddressableAccount):
             raise Exception(f"SerumEventQueue account not found at address '{address}'")
         return SerumEventQueue.parse(account_info)
 
+    @property
+    def accounts_to_crank(self) -> typing.Sequence[PublicKey]:
+        to_crank: typing.List[PublicKey] = []
+        for event_to_crank in self.unprocessed_events:
+            to_crank += [event_to_crank.public_key]
+
+        seen = []
+        distinct = []
+        for account in to_crank:
+            account_str = account.to_base58()
+            if account_str not in seen:
+                distinct += [account]
+                seen += [account_str]
+
+        return distinct
+
     @property
     def capacity(self) -> int:
         return len(self.unprocessed_events) + len(self.processed_events)
